import sys
import fileinput
import math

version = int(sys.version[0])

if version == 3:
  import tkinter as tk
  from tkinter import Tk
  from tkinter.filedialog import askopenfilename
  from tkinter.filedialog import asksaveasfilename
  from tkinter import ttk

else:
  import Tkinter as tk
  from Tkinter import *
  import tkFileDialog
  from tkFileDialog import askopenfilename
  from tkFileDialog import asksaveasfilename
  import ttk

#=================
def inFile_open():
#=================
  global inFile
  inFile = askopenfilename(title="Select the GSS Export file...",initialfile=inFile) # show an "Open" dialog box and return the path to the selected file
  if inFile != "":
    ent_inFile.delete(0,tk.END)
    ent_inFile.insert(0,inFile)
  #print ("Input File:", inFile)

#=================
def update_inFile_edit():
#=================
  global inFile
  inFile = ent_inFile.get()

#=================
def event_inFile_edit(event):
#=================
  update_inFile_edit()

#=================
def outFile_open():
#=================
  global outFile
  outFile = asksaveasfilename(title="Select the output file...",initialfile=outFile) # show an "Open" dialog box and return the path to the selected file
  if outFile != "":
    ent_outFile.delete(0,tk.END)
    ent_outFile.insert(0,inFile)
  #print ("Output File:", outFile)

#=================
def update_outFile_edit():
#=================
  global outFile
  outFile = ent_outFile.get()

#=================
def event_outFile_edit(event):
#=================
  update_outFile_edit()

#=================
def update_coords():
#=================
  global coordsType
  if coordsRb.get() == 1:
    coordsType = "3"
    lbl_invertZ["fg"] = "black"
  elif coordsRb.get() == 2:
    coordsType = "2"
    lbl_invertZ["fg"] = "gray"
  #print ("Coordinates:", coordsRb.get(), coordsType)
  
#=================
def update_units():
#=================
  global unitsType
  if unitsRb.get() == 1:
    unitsType = "f"
  elif unitsRb.get() == 2:
    unitsType = "m"
  #print ("Units:", unitsRb.get(), unitsType)

#=================
def update_degrees():
#=================
  global degreesType
  if degreesRb.get() == 1:
    degreesType = "dd"
  elif degreesRb.get() == 2:
    degreesType = "ddm"
  elif degreesRb.get() == 3:
    degreesType = "dms"
  #print ("Degrees:", degreesRb.get(), degreesType)

#=================
def update_minimumDistance():
#=================
  global minimumDistance
  minimumDistance = ent_minimumDistance.get()
  #print ("Min Dist:", minimumDistance)

#=================
def event_minimumDistance(event):
#=================
  update_minimumDistance()
  
#=================
def update_columns():
#=================
  global columns
  columns = ent_columns.get()
  #print ("Columns:", columns)

#=================
def event_columns(event):
#=================
  update_columns()

#=================
def process():
#=================
  global lbl_rows
  global lbl_points

  update_inFile_edit()
  update_outFile_edit()
  update_minimumDistance()
  update_columns()
  
  lbl_rowsData["text"] = "________"
  lbl_pointsData["text"] = "________"
  gss_to_3d.update_idletasks()
  
  headerRows = 5 - 1 #subtract 1 to account for row 0
  metersPerDegree = 111000
  feetPerDegree = 364173
  if unitsType == "m":
    units = metersPerDegree
  if unitsType == "f":
    units = feetPerDegree

  try:
    minimumDistanceFloat = float(minimumDistance)
  except:
    print(minimumDistance)
    sys.exit("Configuration File Error:  Invalid minimumDistance Value (" + minimumDistance + ")!")

  columnChoice = columns.strip(" ,").split(",")

  lon = lat = 0
  z = 1000000
  rows = points = 0
  outColumns = []
  if icb.get() == 0:
    invert = 1
  else:
    invert = -1

  f = open(outFile, "w")
  for line in fileinput.input(inFile):
    if rows > headerRows:
      dataColumns = line.split(",")
      for column in range(len(columnChoice)):
        outColumns.append(dataColumns[int(columnChoice[column])].strip())
      if degreesType == "ddm":
        lon = float(outColumns[0])
        lat = float(outColumns[1])
        lon = lonD + lonM * 60
        lat = latD + latM * 60
      if degreesType == "dms":
        lon = float(outColumns[0])
        lat = float(outColumns[1])
        lon = lonD + lonM * 60 + LonS * 60
        lat = latD + latM * 60 + latS * 60
      # convert degrees to distance
      dE = (float(outColumns[0]) - lon) * units * (90 - abs(lon)) / 90
      dN = (float(outColumns[1]) - lat) * units
      if coordsType == "3":
        dZ = invert * float(outColumns[2]) - z
        distance = math.sqrt(dE*dE + dN*dN + dZ*dZ)
      else:
        distance = math.sqrt(dE*dE + dN*dN)
      if distance > minimumDistanceFloat:
        #keep this point, otherwise skip it
        lon = float(outColumns[0])
        lat = float(outColumns[1])
        point = str(lon) + "," + str(lat)
        if coordsType == "3":
          z = invert * float(outColumns[2])
          point = point + "," + str(z)
        extra = int(coordsType)
        while extra < len(outColumns):
          point = point + "," + outColumns[extra]
          extra += 1
        point = point + "\n"
        f.write(point)
        points += 1
      if version > 2:
        outColumns.clear()
      else:
        del outColumns[:]
    rows += 1

  #update results
  lbl_rowsData["text"] = "    " + str(rows)
  lbl_pointsData["text"] = "    " + str(points)

#=================
def help():
#=================
  win_help = tk.Tk()
  win_help.title("GSS to 3D Help")
  lbl_help = tk.Label(master=win_help,text=
    "GSS to 3D formats a GSS export file for input to 3D."
    "\n\nPrimary input is GSS export in CSV format and information about which columns to use for output."
    "\n\nInput order is flexible - you defined which columns hold X, Y and Z values."
    "\n\nColumn numbers start at 0 in the input file."
    "\n\nOutput order is Longitude (East or X), Latitude (North or Y) and Z (Depth or Altitude)"
    "\nand any additional column(s) you select to include with the output."
    "\n\nZ is not used for distance moved calculations."
    "\n\nProgram input defaults can be changed by editing gss-3d.cfg"
    "\nLine 1: Input file to read from."
    "\nLine 2: Output file to write to."
    "\nLine 3: Coordinates Type - One of: [ 3 | 2 ]"
    "\n                3 = 3-D; 2 = 2-D"
    "\nLine 4: Invert Z value - One of: [ - | (any other character) ]"
    "\n                If set to \"-\" Z values are multiplied by minus one (-1) to invert the sign."  
    "\nLine 5: Units Type - One of: [ M | F ]"
    "\n                M = Meters; F = Feet"
    "\nLine 6: Degrees Format - One of: [ DD | DDM | DMS ]"
    "\n                DD = Decimal Degrees; DDM = Degrees Decimal Minutes; DMS = Degrees Minutes Seconds" 
    "\nLine 7: Minimum Distance Threshold."
    "\n                Amount the vehicle has to move from a previous point to add a new point."
    "\nLine 8: Column number of the East or X column of the input file to use in output."
    "\nLine 9: Column number of the North or Y column of the input file to use in output."
    "\nLine 10: Column number of the Z column of the input file to use in ouput (not used for 2D)."
    "\nAdditional lines can be added for additional columns desired in the output."
    "\n\nContact VideoRay Customer Care for assistance:  support@videoray.com"
    "\n\n" u'\N{COPYRIGHT SIGN}' + " 2020, VideoRay LLC"
    ,justify=tk.LEFT)
  btn_close=tk.Button(master=win_help,text="Close",font="arial 10 bold",command=win_help.destroy)
  lbl_help.pack(padx=10,pady=10)
  btn_close.pack(pady=10)
  
#=================
def exit():
#=================
  sys.exit()

#begin processing
configurationFile = "gss-3d.cfg"

columnsBase = 7 #rows before column data

columnCount = 0
columnChoice = []

# read configuration file
for line in fileinput.input(configurationFile):
  columnChoice.append(line.strip())
  columnCount += 1
inFile = columnChoice[0].strip()
outFile = columnChoice[1].strip()
coordsType = columnChoice[2].strip()[0:1]
invertZ = columnChoice[3].strip()[0:1]
unitsType = columnChoice[4].strip()[0:1].lower()
degreesType = columnChoice[5].strip().lower()
minimumDistance = columnChoice[6]
columnsMax = columnCount - columnsBase
if columnsMax < int(coordsType):
  sys.exit("Configuration File Error:  Not enough rows!")
try:
  int(columnChoice[0 + columnsBase])
except:
  sys.exit("Configuration File Error:  Invalid Column Value for First column!")
try:
  int(columnChoice[1 + columnsBase])
except:
  sys.exit("Configuration File Error:  Invalid Column Value for Second column!")
columns = columnChoice[0 + columnsBase] + "," + columnChoice[1 + columnsBase]
columnsActive = 2

while columnsActive < columnsMax:
  try:
    int(columnChoice[columnsActive + columnsBase])
  except:
    sys.exit("Configuration File Error:  Invalid Column Value for column " + str(columnsActive + 1) + "!")
  columns = columns + "," + columnChoice[columnsActive + columnsBase]
  columnsActive += 1

#create window
gss_to_3d = tk.Tk()
gss_to_3d.title("GSS to 3D")

#define frames
frm_inFile = tk.Frame()
frm_outFile = tk.Frame()
frm_coordsType = tk.Frame()
frm_unitsType = tk.Frame()
frm_degreesType = tk.Frame()
frm_minimum_distance = tk.Frame()
frm_columns = tk.Frame()
frm_actions = tk.Frame()
frm_rows = tk.Frame()
frm_points = tk.Frame()

#define widgets
lbl_title = tk.Label(text="Format GSS Export for 3D Input",font="arial 12 bold")

lbl_inFile = tk.Label(master=frm_inFile,text="Read from:",height=2)
ent_inFile = tk.Entry(master=frm_inFile,width=60)
btn_inFile = tk.Button(master=frm_inFile,text="Select", command=inFile_open)

lbl_outFile = tk.Label(master=frm_outFile,text="Write to:    ", height=2)
ent_outFile = tk.Entry(master=frm_outFile,width=60)
btn_outFile = tk.Button(master=frm_outFile,text="Select", command=outFile_open)

lbl_coordsType = tk.Label(master=frm_coordsType,text="Coordinates Type:",font="arial 8 bold")
coordsRb = tk.IntVar()
crb1 = tk.Radiobutton(master=frm_coordsType,text="3-D",variable=coordsRb,value=1,command=update_coords)
crb2 = tk.Radiobutton(master=frm_coordsType,text="2-D",variable=coordsRb,value=2,command=update_coords)

lbl_invertZ = tk.Label(master=frm_coordsType,text="Invert Z:",font="arial 8 bold")
icb = tk.IntVar()
cb_invertZ = tk.Checkbutton(master=frm_coordsType,variable=icb)

lbl_unitsType = tk.Label(master=frm_unitsType,text="Units Type:",font="arial 8 bold")
unitsRb = tk.IntVar()
rb1 = tk.Radiobutton(master=frm_unitsType,text="Feet",variable=unitsRb,value=1,command=update_units)
rb2 = tk.Radiobutton(master=frm_unitsType,text="Meters",variable=unitsRb,value=2,command=update_units)

lbl_degreesType = tk.Label(master=frm_degreesType,text="Degrees Format:",font="arial 8 bold")
degreesRb = tk.IntVar()
drb1 = tk.Radiobutton(master=frm_degreesType,text="D.DD",variable=degreesRb,value=1,command=update_degrees)
drb2 = tk.Radiobutton(master=frm_degreesType,text="D M.MM",variable=degreesRb,value=2,command=update_degrees)
drb3 = tk.Radiobutton(master=frm_degreesType,text="D M S ",variable=degreesRb,value=3,command=update_degrees)

lbl_minimumDistance = tk.Label(master=frm_minimum_distance,text="Minimum Distance Threshold:         ")
ent_minimumDistance = tk.Entry(master=frm_minimum_distance,width=10)

lbl_columns = tk.Label(master=frm_columns,text="Columns to Use and Output Order:")
ent_columns = tk.Entry(master=frm_columns,width=10)

btn_help = tk.Button(master=frm_actions,text="Help",font="arial 10 bold",command=help)

btn_process = tk.Button(master=frm_actions,text="Process",font="arial 10 bold",command=process)

btn_exit = tk.Button(master=frm_actions,text="Exit",font="arial 10 bold",command=exit)

sep_results = ttk.Separator(orient="horizontal")

lbl_results = tk.Label(text="Results",font="arial 12 bold")

lbl_rows = tk.Label(master=frm_rows,text="Rows of Data Read:")
lbl_rowsData = tk.Label(master=frm_rows,text="________")

lbl_points = tk.Label(master=frm_points,text="Points Generated:  ")
lbl_pointsData = tk.Label(master=frm_points,text= "________")

lbl_copyright = tk.Label(text=u'\N{COPYRIGHT SIGN}' + " 2020, VideoRay LLC")

#update widget values with configuration file information
ent_inFile.insert(0,inFile)
ent_inFile.bind("<FocusOut>",event_inFile_edit)

ent_outFile.insert(0,outFile)
ent_outFile.bind("<FocusOut>",event_outFile_edit)

if invertZ == "-":
  icb.set(1)
else:
  icb.set(0)

if coordsType == "3":
  coordsRb.set(1)
elif coordsType == "2":
  coordsRb.set(2)
  icb.set(0)
  lbl_invertZ["fg"] = "gray"
else:
  sys.exit("Configuration File Error:  Invalid Coordinates Type (" + coordsType + ")!")

if unitsType == "f":
  unitsRb.set(1)
elif unitsType == "m":
  unitsRb.set(2)
else:
  sys.exit("Configuration File Error:  Invalid Units Type (" + unitsType + ")!")
  
if degreesType == "dd":
  degreesRb.set(1)
elif degreesType == "ddm":
  degreesRb.set(2)
elif degreesType == "dms":
  degreesRb.set(2)
else:
  sys.exit("Configuration File Error:  Invalid Degrees Format (" + degreesType + ")!")

try:
  float(minimumDistance)
except:
  print(minimumDistance)
  sys.exit("Configuration File Error:  Invalid minimumDistance Value (" + minimumDistance + ")!")
ent_minimumDistance.insert(0,str(minimumDistance))
ent_minimumDistance.bind("<FocusOut>",event_minimumDistance)

ent_columns.insert(0,columns)
ent_columns.bind("<FocusOut>",event_columns)

#display window 
lbl_title.pack(pady=2)

lbl_inFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
ent_inFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
btn_inFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_inFile.pack(pady=2)

lbl_outFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
ent_outFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
btn_outFile.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_outFile.pack()

lbl_coordsType.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
crb1.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
crb2.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_coordsType.pack()

lbl_invertZ.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
cb_invertZ.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)

lbl_unitsType.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
rb1.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
rb2.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_unitsType.pack()

lbl_degreesType.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
drb1.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
drb2.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
drb3.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_degreesType.pack()

lbl_minimumDistance.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
ent_minimumDistance.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_minimum_distance.pack()

lbl_columns.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
ent_columns.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_columns.pack()

btn_help.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=12)
btn_process.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=12)
btn_exit.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=12)
frm_actions.pack()

sep_results.pack(fill='x',pady=2)

lbl_results.pack(pady=2)

lbl_rows.pack(fill=tk.BOTH,side=tk.LEFT,padx=10,pady=2)
lbl_rowsData.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_rows.pack()

lbl_points.pack(fill=tk.BOTH,side=tk.LEFT,padx=10,pady=2)
lbl_pointsData.pack(fill=tk.BOTH,side=tk.LEFT,padx=5,pady=2)
frm_points.pack()

lbl_copyright.pack()

#start event loop
gss_to_3d.mainloop()
